#!/bin/sh
#
# Startup script for openvpn server
#

CONF_DIR="/usr/syno/etc/synovpn/openvpn"
USER_CONF="openvpn.conf.user"
USER_CONF_TMP_FILE="/tmp/run.openvpn.conf.user"
OPENVPN_CONF="openvpn.conf"

if [ -e ${CONF_DIR}/${USER_CONF} ]; then
	OPENVPN_CONF=${USER_CONF}
fi

SYNOVPN_CONF="/usr/syno/etc/synovpn/synovpn.conf"
LOCALIP=`grep "^server" ${CONF_DIR}/${OPENVPN_CONF} | cut -d' ' -f 2`
NET_IF=`/bin/get_key_value ${SYNOVPN_CONF} vpninterface`

case "$1" in
  start)
        Run=`/bin/get_key_value ${SYNOVPN_CONF} runopenvpn`
	if [ "yes" != "$Run" ]; then
		exit 0;
	fi
                                        
        if [ -n "`pidof openvpn`" ]; then
		echo "Already running"
		return
	fi
                                      
	echo 1 > /proc/sys/net/ipv4/ip_forward

	for i in x_tables.ko ip_tables.ko iptable_filter.ko nf_conntrack.ko \
		nf_defrag_ipv4.ko nf_conntrack_ipv4.ko nf_nat.ko \
		iptable_nat.ko ipt_REDIRECT.ko xt_multiport.ko xt_tcpudp.ko \
		xt_state.ko ipt_MASQUERADE.ko
	do
		/sbin/insmod /lib/modules/$i
	done

	# Make device if not present (not devfs)
	if ( [ ! -c /dev/net/tun ] ) then
  		# Make /dev/net directory if needed
  		if ( [ ! -d /dev/net ] ) then
        		mkdir -m 755 /dev/net
  		fi
  		mknod /dev/net/tun c 10 200
	fi
	
	if ( !(lsmod | grep -q "^tun") ); then
        	insmod /lib/modules/tun.ko
	fi

	# create link for liblzo
        if [ ! -e /lib/liblzo2.so.2 ]; then
		ln -sf /usr/local/synovpn/lib/liblzo2.so.2.0.0 /lib/liblzo2.so.2
	fi

        # create links : ca.crt, server.crt, and server.key of ssl
        if [ ! -e /usr/local/synovpn/etc/openvpn/keys/ca.crt ]; then
                ln -sf /usr/syno/etc/synovpn/openvpn/keys/ca.crt /usr/local/synovpn/etc/openvpn/keys/
                ln -sf /usr/syno/etc/synovpn/openvpn/keys/server.crt /usr/local/synovpn/etc/openvpn/keys/
                ln -sf /usr/syno/etc/synovpn/openvpn/keys/server.key /usr/local/synovpn/etc/openvpn/keys/
        fi

	# start radius server
	/usr/local/synovpn/scripts/radiusd.sh start

        echo "Starting openvpn ..."
	/usr/local/synovpn/sbin/openvpn --daemon --cd ${CONF_DIR} --config ${OPENVPN_CONF} 

	/sbin/iptables -t nat -A POSTROUTING -s ${LOCALIP}/24 -o ${NET_IF} -j MASQUERADE

	#touch a temp file for UI check
	if [ ${OPENVPN_CONF} == ${USER_CONF} ]; then
		/bin/touch ${USER_CONF_TMP_FILE} 2>/dev/null
	elif [ -e ${USER_CONF_TMP_FILE} ]; then
		/bin/rm -rf ${USER_CONF_TMP_FILE} 2>/dev/null
	fi
        
        ;;
  stop)
        /sbin/iptables -t nat -D POSTROUTING -s ${LOCALIP}/24 -o ${NET_IF} -j MASQUERADE

        echo "Stopping openvpn ..."
        if [ -n "`/bin/pidof openvpn`" ]; then
    		/usr/bin/killall openvpn 2>/dev/null
	fi

	sleep 2	
	/sbin/rmmod tun

	/sbin/rmmod ipt_MASQUERADE xt_state xt_tcpudp xt_multiport \
		ipt_REDIRECT iptable_nat nf_nat nf_conntrack_ipv4 \
		nf_defrag_ipv4 nf_conntrack iptable_filter ip_tables x_tables

	/usr/local/synovpn/scripts/radiusd.sh stop

	#delete USER_CONF_TMP_FILE
	if [ -e ${USER_CONF_TMP_FILE} ]; then
		/bin/rm -rf ${USER_CONF_TMP_FILE} 2>/dev/null
	fi

	;;
  restart)
        sh $0 stop quiet
        sleep 3
        sh $0 start
        ;;
  *)
        echo "Usage: $0 {start|stop|restart}"
        exit 1
esac

exit 0

# [EOF]



